@@ -39,8 +39,8 @@ void Bank::open_stream(int i)
     buffered_file[i]->stream = gzopen(buffered_file[i]->fname,"r");
     if (buffered_file[i]->stream == NULL)
     {
-        printf("error opening file: %s\n",buffered_file[i]->fname);
-        exit(1);
+       fprintf(stderr, "\nERROR: cannot open reads file %s, exit\n",buffered_file[i]->fname);
+       exit(1);
     }
 }
 // and close it
