@@ -61,7 +61,7 @@ from operator import itemgetter
 from subprocess import Popen, PIPE, STDOUT
 
 
-TIMEOUT=900
+TIMEOUT=3600
 
 unfinished_bccs = []
 num_snps = {}
@@ -73,10 +73,11 @@ class Command(object): # deprecated in the future with Python3
     def target(self, **kwargs):
         self.process = Popen(kwargs["args"], stdout=kwargs["stdout"], stderr=PIPE)
         com = self.process.communicate()
-        if com[0]: print com[0]
+        if com[0] and (kwargs["verbose"] or self.process.returncode != 0): 
+            print com[0]
 
         # Prints stderr that was piped by Popen 
-        if com[1] and kwargs["verbose"]: 
+        if com[1] and (kwargs["verbose"] or self.process.returncode != 0):
             print com[1] 
         
     def run(self, command_line, out_file = "", mode = 'w', verbose = False, timeout = sys.maxint):
@@ -111,12 +112,9 @@ class Command(object): # deprecated in the future with Python3
         elif self.process.returncode == 15:
             print >> sys.stderr, "\n\t\t *** Maximum number of bubbles reached! ***\n" 
         elif self.process.returncode != 0:
-            print >> sys.stderr, "Problem with " + command_line  + " : this may be due to memory issues. try setting your stack size as unlimited using: ulimit -s unlimited "
-
+            print >> sys.stderr, "Problem with " + command_line.split()[0] 
             sys.exit(self.process.returncode)
-        else: 
-            if verbose:       
-                print "Done!"
+
             
 def mkdirTmp(tmpdir=None):
     if not tmpdir:
@@ -143,7 +141,8 @@ def to_file(readfiles, filename = "tmp"):
 
 # Run debruijn graph construction
 def build_graph(KISSPLICE_INSTDIR, workdir, readfiles, kval, graphfile, min_cov, genome_size, verbose = False):    
-    print "Building de Bruijn graph..."
+    print "--> Building de Bruijn graph..."
+    print "Graph will be written in "+graphfile+".[edges/nodes]"
     all_read_files = workdir + "/all_read_filenames"
     to_file(readfiles, all_read_files)
     command_line = KISSPLICE_INSTDIR+"/ks_debruijn4 " + all_read_files+" "+str(kval)+" "+str(min_cov)+" "+str(genome_size)+" "+ workdir + "/graph --kissplice --4bloom"
@@ -151,10 +150,11 @@ def build_graph(KISSPLICE_INSTDIR, workdir, readfiles, kval, graphfile, min_cov,
     shutil.move(workdir+"/graph.edges", graphfile + ".edges")
     shutil.move(workdir+"/graph.nodes", graphfile + ".nodes")
     shutil.move(workdir+"/graph.solid_kmers_binary_with_count", graphfile + ".solid_kmers_binary_with_count")
+    print "--> Done!"
 
 #Run error_removal for the graph (overwrite edge file)
 def error_removal(KISSPLICE_INSTDIR, workdir, kval, graphfile, cutoff, verbose = False):
-    print "\n\nRemoving sequencing errors..."
+    print "\n--> Removing sequencing errors..."
 
     # convert binary count file to text 
     if os.path.isfile(graphfile+".solid_kmers_binary_with_count"):
@@ -179,13 +179,14 @@ def error_removal(KISSPLICE_INSTDIR, workdir, kval, graphfile, cutoff, verbose =
         subprocessLauncher(command_line, verbose=verbose)
     else:
         print "k-mer count file not found! Skipping error removal step."
+    print "--> Done!"
 
 
 #Run the modules on the graph
 def run_modules(KISSPLICE_INSTDIR, workdir, graphfile, kval, cutoff, verbose = False, output_context = False, exec_error_removal = False):
     if not os.path.exists(workdir+"/bcc"):
         os.mkdir(workdir+"/bcc")    
-    print "\nFinding the BCCs..."
+    print "\n--> Finding the BCCs..."
     
     edge_suffix = ".edges"
     if exec_error_removal:
@@ -195,7 +196,10 @@ def run_modules(KISSPLICE_INSTDIR, workdir, graphfile, kval, cutoff, verbose = F
 
     if output_context:
         command_line += " --output-context"
-    subprocessLauncher(command_line, verbose=verbose)
+    return_code = subprocessLauncher(command_line, verbose=verbose)
+    if (return_code != 0):
+        print "This may be due to a stack overflow. Try setting your stack size as unlimited using: ulimit -s unlimited."
+    print "--> Done!"
   
 def count_alreadyfoundSNPs(workdir): 
     global num_snps 
@@ -206,8 +210,8 @@ def count_alreadyfoundSNPs(workdir):
         num_snps[info[0]] = int(info[1])  
     info_snp_file.close()
  
-def enumerate_all_bubbles(KISSPLICE_INSTDIR, workdir, outdir, kval, output_snps, min_edit_dist, max_cycles, UL_MAX, LL_MAX, LL_MIN, timeout, nbprocs=1, verbose = False, output_context = False, output_path = False):
-    print "Enumerating all bubbles..."
+def enumerate_all_bubbles(KISSPLICE_INSTDIR, workdir, outdir, kval, bval, output_snps, min_edit_dist, max_cycles, UL_MAX, LL_MAX, LL_MIN, timeout, nbprocs=1, verbose = False, output_context = False, output_path = False):
+    print "\n--> Enumerating all bubbles..."
 
     if os.path.isfile(workdir+"/all_bcc_type0_"+str(kval)):
       os.remove(workdir+"/all_bcc_type0_"+str(kval))
@@ -234,7 +238,7 @@ def enumerate_all_bubbles(KISSPLICE_INSTDIR, workdir, outdir, kval, output_snps,
     pool = multiprocessing.Pool(nbprocs)
     TASKS = []
     for i in bccnumorderedbysize:
-        TASKS +=  [(enumerate_bubbles_core, i, KISSPLICE_INSTDIR, workdir, outdir, kval, output_snps, min_edit_dist, max_cycles, UL_MAX, LL_MAX, LL_MIN, timeout, verbose, output_context, output_path)]
+        TASKS +=  [(enumerate_bubbles_core, i, KISSPLICE_INSTDIR, workdir, outdir, kval, bval, output_snps, min_edit_dist, max_cycles, UL_MAX, LL_MAX, LL_MIN, timeout, verbose, output_context, output_path)]
 
     imap_unordered_it = pool.imap_unordered(eval_func_tuple, TASKS, 1)
     
@@ -270,12 +274,13 @@ def enumerate_all_bubbles(KISSPLICE_INSTDIR, workdir, outdir, kval, output_snps,
     size1234 = sum(1 for line in f) 
     f.close()
     n_bubbles = (size0 + size1234)/4
-    print "Total number of bubbles found: ", n_bubbles  
+    print "Total number of bubbles found: ", n_bubbles 
+    print "--> Done!"
 
 
 
 
-def enumerate_bubbles_core(i, KISSPLICE_INSTDIR, workdir, outdir, kval, output_snps, min_edit_dist, max_cycles, UL_MAX, LL_MAX, LL_MIN, timeout, verbose = False, output_context = False, output_path = False):  
+def enumerate_bubbles_core(i, KISSPLICE_INSTDIR, workdir, outdir, kval, bval, output_snps, min_edit_dist, max_cycles, UL_MAX, LL_MAX, LL_MIN, timeout, verbose = False, output_context = False, output_path = False):  
     if verbose:
       print "@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@"
       print "Enumerating bubbles in biconnected component "+str(i)
@@ -303,22 +308,24 @@ def enumerate_bubbles_core(i, KISSPLICE_INSTDIR, workdir, outdir, kval, output_s
       command_line += " -c"
     if output_path:
       command_line += " -p"
+    if bval is not None:
+      command_line += " -b" + str(bval)
 
     process_returncode = subprocessLauncher(command_line, verbose=verbose, timeout=timeout)
 
-    # Store the bcc number if it timed out OR the maximum number of bubbles was reached 
+    # Store the bcc number if it timed out (return code -15) OR the maximum number of bubbles was reached (return code 15) 
     if process_returncode == -15 or process_returncode == 15:
         flag = i
-    # Only append the results if the enumeration is complete.
-    else: 
-        command_line_type0 = KISSPLICE_INSTDIR+"/ks_clean_duplicates " + workdir + "/bcc/tmp_bcc_sequences_" + str(kval) +"_"+multiprocessing.current_process().name+ "_type0.fa"
-        command_line_type1234 = KISSPLICE_INSTDIR+"/ks_clean_duplicates " + workdir + "/bcc/tmp_bcc_sequences_" + str(kval) +"_"+multiprocessing.current_process().name+ "_type1234.fa"
-        subprocessLauncher(command_line_type0, workdir+"/tmp_all_bcc_type0_"+str(kval)+"_"+multiprocessing.current_process().name, 'a', verbose=verbose) # append ALL BCC IN THE SAME FILE
-        subprocessLauncher(command_line_type1234, workdir+"/tmp_all_bcc_type1234_"+str(kval)+"_"+multiprocessing.current_process().name, 'a', verbose=verbose) # append ALL BCC IN THE SAME FILE
-
-        if output_path:
-            command_line = "cat "+workdir+"/bcc/tmp_bcc_sequences_" + str(kval) +"_"+multiprocessing.current_process().name+ ".path" 
-            subprocessLauncher(command_line, workdir+"/tmp_all_paths_bcc_"+str(kval)+"_"+multiprocessing.current_process().name, 'a', verbose=verbose) # append ALL BCC IN THE SAME FILE 
+    
+    # Always append the results if the enumeration, even when it's incomplete.
+    command_line_type0 = KISSPLICE_INSTDIR+"/ks_clean_duplicates " + workdir + "/bcc/tmp_bcc_sequences_" + str(kval) +"_"+multiprocessing.current_process().name+ "_type0.fa"
+    command_line_type1234 = KISSPLICE_INSTDIR+"/ks_clean_duplicates " + workdir + "/bcc/tmp_bcc_sequences_" + str(kval) +"_"+multiprocessing.current_process().name+ "_type1234.fa"
+    subprocessLauncher(command_line_type0, workdir+"/tmp_all_bcc_type0_"+str(kval)+"_"+multiprocessing.current_process().name, 'a', verbose=verbose) # append ALL BCC IN THE SAME FILE
+    subprocessLauncher(command_line_type1234, workdir+"/tmp_all_bcc_type1234_"+str(kval)+"_"+multiprocessing.current_process().name, 'a', verbose=verbose) # append ALL BCC IN THE SAME FILE
+    
+    if output_path:
+        command_line = "cat "+workdir+"/bcc/tmp_bcc_sequences_" + str(kval) +"_"+multiprocessing.current_process().name+ ".path" 
+        subprocessLauncher(command_line, workdir+"/tmp_all_paths_bcc_"+str(kval)+"_"+multiprocessing.current_process().name, 'a', verbose=verbose) # append ALL BCC IN THE SAME FILE 
  
     if verbose:          
       print "@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@\n"  
@@ -329,7 +336,7 @@ def eval_func_tuple(f_args):
     return f_args[0](*f_args[1:])       
         
 def check_read_coverage_and_sort_all_bubbles(KISSPLICE_INSTDIR, readfiles, workdir, outdir, kval, output_snps, infix_name,  countsMethods, minOverlap, substitutions, verbose = False):
-    print "Compute read coherence and coverage"
+    print "\n--> Computing read coherence and coverage..."
     if output_snps:
         destinationSNPS = open(workdir+"/all_bcc_type0_"+str(kval), 'ab') ## THE UNIQUE FILE ALSO CONTAINS SNPS
         shutil.copyfileobj(open(workdir+"/bcc/graph_all_log_bcc", 'rb'), destinationSNPS)
@@ -347,7 +354,9 @@ def check_read_coverage_and_sort_all_bubbles(KISSPLICE_INSTDIR, readfiles, workd
     commandLineCat = "cat " +  workdir+"/uncoherentType1234.fa " + workdir+"/uncoherentType0.fa "
     subprocessLauncher(commandLineCat, workdir + "/uncoherent.fa", "a", verbose=verbose )
     
-    print "Sorting all bubbles..."
+    print "--> Done!"
+
+    print "\n--> Sorting all bubbles..."
 
     nb = [0]*5# counter of number of events of each type found
     cofilel = []
@@ -384,6 +393,9 @@ def check_read_coverage_and_sort_all_bubbles(KISSPLICE_INSTDIR, readfiles, workd
         uncofileout.write(event.split()[-1].replace(';','\n')+"\n")
     uncofile.close()
     uncofileout.close()
+    
+    print "--> Done!"
+
     print "\n\n \t\t ******** We are done, final coherent results are in files "+outdir+"/results_"+infix_name+"_coherents_type_*.fa ********** " 
     print " \t\t ******** All non read coherent results are in files "+outdir+"/results_"+infix_name+"_uncoherent.fa ****** \n\n"
     return nb
@@ -468,6 +480,7 @@ def main():
                       help="input fasta/q read files or compressed (.gz) fasta/q files (mutiple, such as \"-r file1 -r file2...\") ") 
   parser.add_argument('-k', action="store", dest="kval", type=int, default=41,
                       help="k-mer size (default=41)")    
+  parser.add_argument('-b', action="store", dest="bval", type=int, default=5, help="maximum number of branching nodes (default 5)")
   parser.add_argument('-l', action="store", dest="llmax", type=int, default=0,
                       help="maximal length of the shorter path (default: 2k-1)")    
   parser.add_argument('-m', action = "store", dest = "LL_MIN", default = 0, help = "minimum length of the shorter path (default 2k-8)")
@@ -481,24 +494,24 @@ def main():
                       help="specific directory (absolute path) where to build temporary files (default temporary directory otherwise)")
   parser.add_argument('-t', action="store", dest="nbprocs", type=int, default=1,
                       help="number of cores (must be <= number of physical cores)")
-  parser.add_argument('-s', action="store_true", dest="nooutput_snps", help="Will not output SNPs (save time)")  
+  parser.add_argument('-s', action="store_true", dest="output_snps", help="Will output SNPs (increases the running time)")  
   parser.add_argument('-v', action="store_true", dest="verbose", help="Verbose mode")   
-  parser.add_argument('-u', action="store", dest="keep_ubccs", type=int, default=1, help=" 0 or 1. 1 to keep the nodes/edges file for unfinished bccs, 0 to discard (default:1) ")
+  parser.add_argument('-u', action="store_true", dest="keep_ubccs", help="keep the nodes/edges file for unfinished bccs")
   parser.add_argument('-c',  action = "store", dest = "min_cov", default = 2, help="an integer, k-mers present strictly less than this number of times in the dataset will be discarded (default 2)")
   parser.add_argument('-C',  action = "store", dest = "min_relative_cov", default = 0.02, help="a percentage from [0,1), edges with relative coverage below this number are removed (default 0.02)")
   parser.add_argument('-z', action="store", dest="genome_size", type=int, default=1000000000,
-                      help="estimated genome/transcriptome size (default = 1000000000)")    
+                      help="estimated genome/transcriptome size (default = 1G)")    
   parser.add_argument('-e',  action = "store", dest = "min_edit_dist", default = 3, 
                       help="edit distance threshold, if the two sequences (paths) of a bubble have edit distance smaller than this threshold, the bubble is classified as an inexact repeat (default 3)")
-  parser.add_argument('-y',  action = "store", dest = "max_cycles", default = 10000,
-                       help="maximal number of bubbles enumeration in each bcc. If exceeded, no bubble is output for the bcc (default 10000)")
+  parser.add_argument('-y',  action = "store", dest = "max_cycles", default = 100000000,
+                       help="maximal number of bubbles enumeration in each bcc. If exceeded, no bubble is output for the bcc (default 100M)")
   parser.add_argument('--mismatches',  action = "store", dest = "mism", default = 0, type = int, 
                       help="Maximal number of substitutions authorized between a read and a fragment (for quantification only), default 0. If you increase the mismatch and use --counts think of increasing min_overlap too.")
   parser.add_argument('--counts',  action = "store", dest = "countsMethod", default = "0", help="0,1 or 2 . Changes how the counts will be reported. If 0 (default): total counts, if 1: counts on junctions, if 2: all counts. see User guide for more information ")
   parser.add_argument('--min_overlap',  action = "store", dest = "minOverlap", default = 3, help="Set how many nt must overlap a junction to be counted by --counts option. Default=3. see User guide for more information ")
   parser.add_argument('--timeout', action='store', dest="timeout", default=TIMEOUT,
-                      help="max amount of time (in seconds) spent for enumerating bubbles in each bcc. If exceeded, no bubble is output for the bcc (default 900)")
-  parser.add_argument('--version', action='version', version='%(prog)s 2.1.0') 
+                      help="max amount of time (in seconds) spent for enumerating bubbles in each bcc. If exceeded, no bubble is output for the bcc (default 3600)")
+  parser.add_argument('--version', action='version', version='%(prog)s 2.2.1') 
   parser.add_argument('--output-context', action="store_true", dest="output_context", default = False, help="Will output the maximum non-ambiguous context of a bubble")
   parser.add_argument('--output-path', action="store_true", dest="output_path", default = False, help="Will output the id of the nodes composing the two paths of the bubbles.")
     
@@ -506,17 +519,15 @@ def main():
   # ------------------------------------------------------------------------
   #               Parse and interpret command line arguments
   # ------------------------------------------------------------------------
-  options = parser.parse_args()     
+  options = parser.parse_args()    
 
 
  # ------------------------------------------------------------------------
  #                 Print version and command line
  # ------------------------------------------------------------------------
 
-  print "\nThis is KisSplice, version 2.1.0"
-  print "The command line was:\n"
-  print ' '.join(sys.argv)
-  print "\n"
+  print "\nThis is KisSplice, version 2.2.1\n"
+  print "The command line was:       " + ' '.join(sys.argv)
   # ---------------------------------------------------------- Input options
   readfiles = None
   only_graph = False
@@ -542,13 +553,14 @@ def main():
   if not os.path.exists(outdir):
     os.mkdir(outdir)
   
-  output_snps = True
-  if options.nooutput_snps:
-    output_snps = False
-  if options.verbose:
-    print "Will work on reads contained in files ", readfiles
-    print "Results stored in ", options.out_dir  
-  
+  output_snps = False
+  if options.output_snps:
+    output_snps = True
+#  if options.verbose:
+  print "Using the read files:      ", readfiles
+  print "Results will be stored in: ", os.path.abspath(options.out_dir)  
+  print "\n"
+
   # ------------------------------------------------------------- k-mer size
   kval = options.kval
   if kval %2 == 0:
@@ -597,7 +609,6 @@ def main():
     infix_name += "k" + str(kval)
     if not options.graph_prefix:    
       graphfile = options.out_dir+"/graph_"+infix_name # Output graph file
-      print "Graph will be written in "+graphfile+".[edges/nodes]"
   else:
     infix_name = graphfile.split("/")[-1].split(".")[0] #remove what is before the "/" and what is after the "."
 
@@ -639,7 +650,7 @@ def main():
   #                             Enumerate Bubbles
   # ------------------------------------------------------------------------
   t = time.time()
-  enumerate_all_bubbles(KS_SEC_EXEC_PATH, workdir, outdir, kval, output_snps, min_edit_dist, max_cycles, \
+  enumerate_all_bubbles(KS_SEC_EXEC_PATH, workdir, outdir, kval, options.bval, output_snps, min_edit_dist, max_cycles, \
                         UL_MAX, LL_MAX, LL_MIN, float(options.timeout), options.nbprocs, options.verbose, \
                         options.output_context, options.output_path)
   if options.verbose:
@@ -654,7 +665,7 @@ def main():
   if only_graph:
     nb = sort_all_bubbles(KS_SEC_EXEC_PATH, readfiles, workdir, outdir, kval, output_snps, infix_name, options.verbose)
   else:
-    nb = check_read_coverage_and_sort_all_bubbles(KS_SEC_EXEC_PATH, readfiles, workdir, outdir, kval, output_snps, infix_name, countsMethod,  options.mism, options.verbose)
+    nb = check_read_coverage_and_sort_all_bubbles(KS_SEC_EXEC_PATH, readfiles, workdir, outdir, kval, output_snps, infix_name, countsMethod, minOverlap, options.mism, options.verbose)
   if options.verbose:
     print "Elapsed time: ", (time.time() - t), " seconds\n"
   
@@ -664,7 +675,7 @@ def main():
   # ------------------------------------------------------------------------
   if len(unfinished_bccs) != 0:     
       print "\t\t ******** There were " + str(len(unfinished_bccs)) + " BCCs with unfinished enumeration ********"
-      if options.keep_ubccs == 0 :
+      if not options.keep_ubccs:
           print "\t\t ******** Re-run with `-u` to retrieve the unfinished components ********\n"
       else:  
           save_unfinished_bccs(unfinished_bccs, KS_SEC_EXEC_PATH, workdir, outdir, options.verbose)
@@ -676,7 +687,7 @@ def main():
   if output_snps:
     print "\t\t\t 0: SNPs, Inexact Repeats or sequencing substitution errors, "+str(nb[0])+" found"
   else:
-    print "\t\t\t 0: With -s option, SNPs and sequencing substitution where not searched"
+    print "\t\t\t 0: Run with -s option to also search for SNPs"
   print "\t\t\t 1: Alternative Splicing Events, "+str(nb[1])+" found"
   print "\t\t\t 2: Inexact Tandem Repeats, "+str(nb[2])+" found"
   print "\t\t\t 3: Short Indels (<3nt), "+str(nb[3])+" found"
