@@ -51,6 +51,7 @@
 #include <string>
 #include <vector>
 #include <map>
+#include <set>
 #include <queue>
 #include <unistd.h>
 #include "LabelledCEdge.h"
@@ -71,11 +72,12 @@ int LL_MAX;
 int LL_MIN;
 int MIN_DIST;
 int BUBBLE_COUNT_OFFSET;
+int MAX_BRANCHES;
+
 bool OUTPUT_CONTEXT; 
 bool OUTPUT_SNPS;
 bool OUTPUT_PATH;
 
-
 // OUTPUT  FILE, One for SNPs, One for other type of Bubble
 FILE *seq_output_file_type0;
 FILE *seq_output_file_type1234;
@@ -85,6 +87,7 @@ string comment = "";
 
 vector<char*> seqs;
 int k_value;
+int nb_nodes;
 
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////////
 
@@ -97,6 +100,8 @@ string getSeq(int i)
   return (i < (int)seqs.size()) ? string(seqs[i]) : reverse_complement(string(seqs[i % (int)seqs.size()])); 
 }
 
+
+
 /*!
  * \brief Return the path sequence ( with or without context)
  * \param path: nodes ids path
@@ -131,22 +136,16 @@ string path2seq(vector<int>& path, bool all_nodes, bool output)
   return pseq;
 }
 
-
-
-
 void print_path(FILE* stream, vector<int>& path)
 {
   for (int i = 0; i < (int)path.size(); i++)
   {
     fprintf(stream, " %d", path[i]);
-    fprintf(stream, "\n");
   }
+  fprintf(stream, "\n");
 }
 
-
-
-
-void output_bubble(FILE* stream, string& upper, string& lower, int type, int num, int contextFirst, int contextLast)
+void output_bubble(FILE* stream, string& upper, int upper_b, string& lower, int lower_b, int type, int num, int contextFirst, int contextLast)
 {
 // passing in lower case context
   if(contextFirst != 0 || contextLast != 0)
@@ -188,7 +187,7 @@ void output_bubble(FILE* stream, string& upper, string& lower, int type, int num
 /*
  *  for output node-ids option
  */
-void output_bubble(FILE* stream, vector<int>& upper, vector<int>& lower, int type, int num)
+void output_bubble(FILE* stream, vector<int>& upper, int upper_b, vector<int>& lower, int lower_b, int type, int num)
 {
   if (type==0)
   {
@@ -263,12 +262,20 @@ int classify_bubble(vector<int>& p1_nodes, vector<int>& p2_nodes)
 
 ///////////////////////////////////////////////////////////////////////////////////////////////////////////
 
+set<int> in_sub;
+
+bool contains(set<int> &s, int elem)
+{
+  return s.find(elem) != s.end();
+}
+
+
 /* Used by the heap, as the pair <priority, key>. */
 typedef pair<int, int> ii;
 
 /* Dijsktra's algorithm. Returns the distance (shortest path) from
    source to every node. The results are in dist. */
-void dijkstra(int max_dist, int source, vector<int>& dist, WeightedDigraph& G)
+void dijkstra(int max_dist, int source, map<int,int>& dist, WeightedDigraph& G)
 {
   priority_queue<ii,vector<ii>, greater<ii> > Q; 
 
@@ -285,8 +292,11 @@ void dijkstra(int max_dist, int source, vector<int>& dist, WeightedDigraph& G)
       if (!G.adj_list[v][i].removed)
       {
         int w = G.adj_list[v][i].node, cost = G.adj_list[v][i].cost;
-        if (!G.removed[w])
+        if (!G.removed[w] && contains(in_sub, w))
         {
+	  if (dist.find(w) == dist.end())
+	    dist[w] = MAX_DIST;
+
           if (dist[w] > dist[v] + cost)
           {
             dist[w] = dist[v] + cost;
@@ -300,7 +310,8 @@ void dijkstra(int max_dist, int source, vector<int>& dist, WeightedDigraph& G)
 /* Checks if the distance from s to t is smaller than max_dist */
 bool dist_st(int max_dist, int s, int t, WeightedDigraph& G)
 {
-  vector<int> dist(G.adj_list.size(), MAX_DIST);
+  map<int,int> dist;
+  dist[t] = MAX_DIST;
 
   dijkstra(max_dist, s, dist, G);
 
@@ -312,18 +323,44 @@ bool dist_st(int max_dist, int s, int t, WeightedDigraph& G)
    compatible paths. */
 bool testPairCompatible(int s1, int a1, int s2, int a2, WeightedDigraph& G)
 {
-  vector<int> dist1(G.adj_list.size(), MAX_DIST), dist2(G.adj_list.size(), MAX_DIST);
+  map<int,int> dist1, dist2;
 
   dijkstra(a1, s1, dist1, G);
   dijkstra(a2, s2, dist2, G);
 
-  for (int i = 0; i < (int)G.adj_list.size(); i++)
-    if (dist1[i] <= (a1 + G.node_cost[i]) && dist2[i] <= (a2 + G.node_cost[i]))
-      return true;
+  map<int,int>::iterator it;
+  for (it = dist1.begin(); it != dist1.end(); it++)
+    if (dist2.find(it->first) != dist2.end())
+    {
+      int i = it->first;
+      if (dist1[i] <= (a1 + G.node_cost[i]) && dist2[i] <= (a2 + G.node_cost[i]))
+	return true;
+    }
 
   return false;
 }
 
+/* In-degree is equal to out-degree of the completary node, because
+   it's a de Bruijn graph.*/
+int inDegree(int node, WeightedDigraph& G)
+{
+  if (node < nb_nodes)
+    return G.outDegree(node + nb_nodes);
+  else
+    return G.outDegree(node - nb_nodes);
+}
+
+int nbBranchingNodes(vector<int>& p_nodes, WeightedDigraph& G)
+{
+  int nb = 0;
+  
+  // Disregard the initial and final nodes.
+  for (int i = 1; i < (int)p_nodes.size()-1; i++)
+    if (G.outDegree(p_nodes[i]) != 1 ||  inDegree(p_nodes[i],G) != 1)
+      nb++;
+  return nb;
+}
+
 /*!
  * \brief list all pair of compatible paths (see paper), classify the events found
  * and output them
@@ -343,6 +380,9 @@ bool testPairCompatible(int s1, int a1, int s2, int a2, WeightedDigraph& G)
    paper for a full description. */
 void listPairCompatible(int s1, int a1, int p1, vector<int>& p1_nodes, int s2, int a2, int p2, vector<int>& p2_nodes, WeightedDigraph& G)
 {
+  if (nbBranchingNodes(p1_nodes, G) > MAX_BRANCHES || nbBranchingNodes(p2_nodes, G) > MAX_BRANCHES)
+    return;
+
   if (s1 == s2 && (p1_nodes.size() != 1 || p2_nodes.size() != 1)) 
   {
     if ( p1 >= (beta + G.node_cost[s1]) && p2 >= (beta + G.node_cost[s2]) )
@@ -353,11 +393,13 @@ void listPairCompatible(int s1, int a1, int p1, vector<int>& p1_nodes, int s2, i
       {
         string upper_seq = path2seq(p1_nodes, OUTPUT_CONTEXT, true);
         string lower_seq = path2seq(p2_nodes, OUTPUT_CONTEXT, true);
+	int upper_b = nbBranchingNodes(p1_nodes,G), lower_b = nbBranchingNodes(p2_nodes,G); 
         bool swp = false;
 
         if (upper_seq.size() < lower_seq.size())
         {
           swap(upper_seq, lower_seq);
+	  swap(upper_b, lower_b);
           swp = true;
         }
 
@@ -368,15 +410,15 @@ void listPairCompatible(int s1, int a1, int p1, vector<int>& p1_nodes, int s2, i
           int contextLeft = getSeq(p1_nodes[0]).size() - (k_value );
           int contextRight = getSeq(p1_nodes[(int)p1_nodes.size()-1]).size() - (k_value );
 
-          output_bubble( selection, upper_seq, lower_seq, type, BUBBLE_COUNT_OFFSET + nbBubbles,  contextLeft, contextRight);
+          output_bubble( selection, upper_seq, upper_b, lower_seq, lower_b, type, BUBBLE_COUNT_OFFSET + nbBubbles,  contextLeft, contextRight);
         }
         else
         {
-          output_bubble( selection, upper_seq, lower_seq, type, BUBBLE_COUNT_OFFSET + nbBubbles,  0, 0); // size of the context if null
+          output_bubble( selection, upper_seq, upper_b, lower_seq, lower_b, type, BUBBLE_COUNT_OFFSET + nbBubbles,  0, 0); // size of the context if null
         }
         if (OUTPUT_PATH)
         {
-          output_bubble(path_output_file, swp ? p2_nodes : p1_nodes, swp ? p1_nodes : p2_nodes, type, BUBBLE_COUNT_OFFSET + nbBubbles);
+          output_bubble(path_output_file, swp ? p2_nodes : p1_nodes, upper_b, swp ? p1_nodes : p2_nodes, lower_b, type, BUBBLE_COUNT_OFFSET + nbBubbles);
         }
       }
       nbBubbles++;
@@ -385,16 +427,17 @@ void listPairCompatible(int s1, int a1, int p1, vector<int>& p1_nodes, int s2, i
     }
     return;
   }
+  
   if (G.adjListSz(s1) == 0 && G.adjListSz(s2) == 0)
     return;
-
+  
   int u = (G.adjListSz(s1) != 0) ? s1 : s2;
 
   G.removed[u] = true;
   for (int i = 0; i < (int)G.adj_list[u].size(); i++)
   {
     int v = G.adj_list[u][i].node, cost = G.adj_list[u][i].cost;
-    if (!G.removed[v] && !G.adj_list[u][i].removed)
+    if (!G.removed[v] && !G.adj_list[u][i].removed && contains(in_sub, v))
     {
       if (u == s1 && testPairCompatible(v, a1 - cost, s2, a2, G))
       {
@@ -422,6 +465,44 @@ void listPairCompatible(int s1, int a1, int p1, vector<int>& p1_nodes, int s2, i
   }
 }
 
+
+void bfs(WeightedDigraph &G, int source, set<int> &in_sub)
+{
+  queue<int> Q;
+  map<int, int> dist;
+    
+  Q.push(source);
+  dist[source] = 0;
+  while (!Q.empty())
+  {
+    int u = Q.front();
+    Q.pop();
+
+    in_sub.insert(u);
+    int d = dist[u];
+
+    for (int i = 0; i < (int)G.adj_list[u].size(); i++)
+    {
+      int v = G.adj_list[u][i].node;
+      if (!contains(in_sub, v))
+      {
+	Q.push(v);
+	dist[v] = d + 1;
+      }
+    }  
+    // We don't compress non-branching paths to an edge, but to a
+    // vertex, this means that each non-branching vertex is followed
+    // by to branching vertices. Implying that a path with
+    // MAX_BRANCHES contains at most 2*MAX_BRANCHES+1
+    // vertices. However, we don't count the first and last vertices
+    // (that are always branching), so the length is 2 *
+    // (MAX_BRANCHES+1).
+    if (d > 2*(MAX_BRANCHES+1))
+      return;
+  }  
+  dist.clear();
+}
+
 /* List all bubbles satisfying the path constraints. It does so using
    listPairCompatible function. */
 void listAllBubbles(WeightedDigraph& G, int k_value, int UL_MAX, int LL_MAX, int LL_MIN)
@@ -432,6 +513,9 @@ void listAllBubbles(WeightedDigraph& G, int k_value, int UL_MAX, int LL_MAX, int
 
   for (int v = 0; v < (int)G.adj_list.size(); v++)
   {
+    in_sub.clear();
+    bfs(G, v, in_sub);
+    
     vector<int> p1_nodes, p2_nodes;
 
     p1_nodes.push_back(v); p2_nodes.push_back(v);
@@ -454,11 +538,12 @@ void printSummary( FILE* stream, int num_bubbles )
 
 void printUsageAndExit( char * name )
 { 
-  fprintf( stderr, "Usage: %s infofile contents_file_edges contents_file_nodes basename_edges basename_nodes number_to_read k_value output_prefix edit_distance_threshold comment numbering_offset [-u UL_MAX] [-L LL_MAX] [-l LL_MIN] [-M MAX_BUBBLES] [-s] [-p] [-c]\n", name );
+  fprintf( stderr, "Usage: %s infofile contents_file_edges contents_file_nodes basename_edges basename_nodes number_to_read k_value output_prefix edit_distance_threshold comment numbering_offset [-u UL_MAX] [-L LL_MAX] [-l LL_MIN] [-M MAX_BUBBLES] [-s] [-p] [-c] [-b]\n", name );
   fprintf( stderr, "\t [-u UL_MAX] Maximal length of the upper path of each bubble. Default: 1000\n" );
   fprintf( stderr, "\t [-L LL_MAX] Maximal length of the lower path of each bubble. Default: 2k-1\n" );
   fprintf( stderr, "\t [-l LL_MIN] Minimal length of the lower path of each bubble. Default: 2k-8\n" );
   fprintf( stderr, "\t [-M MAX_BUBBLES] Stop the process after this number of bubbles. Default: 10000\n" );
+  fprintf( stderr, "\t [-b MAX_BRANCHES] Maximum number of branches for each bubble. Default: 5\n");
   exit( EXIT_FAILURE );
 }
 
@@ -482,13 +567,14 @@ int main( int argc, char** argv )
   LL_MIN = 2 * k_value - 8;
   UL_MAX = 1000;
   MAX_BUBBLES = 10000;
+  MAX_BRANCHES = 5;
   OUTPUT_CONTEXT = false;
   OUTPUT_SNPS = true;
   OUTPUT_PATH = false;
   int required_sequence = atoi( argv[6] );
 
   int temoin;
-  while ( (temoin = getopt ( argc-11, &argv[11], "u:L:l:M:scp" )) != -1 ) 
+  while ( (temoin = getopt ( argc-11, &argv[11], "u:L:l:M:b:scp" )) != -1 ) 
   {
     switch ( temoin )
     {
@@ -512,10 +598,15 @@ int main( int argc, char** argv )
       MAX_BUBBLES = atoi( optarg );
       break;
     }
+    case 'b':
+    {
+      MAX_BRANCHES = atoi( optarg );
+      break;
+    }
     case 's':
     {
       OUTPUT_SNPS = false;
-      fprintf(stderr, "Will not output SNPs and sequencing errors\n" );
+      fprintf(stderr, "Will not ouput SNPs and sequencing errors\n" );
       break;
     }
     case 'c':
@@ -536,15 +627,16 @@ int main( int argc, char** argv )
     }
     }
   }
+  fprintf(stdout, "\t Enumerating bubbles with at most %d branching nodes in each path!\n", MAX_BRANCHES);
 
   EdgeLoader edgeloader(allEdges);
   NodeLoader nodeloader(seqs, label);
   bool atleast4nodes = read_edges_and_nodes_withoptimIO<EdgeLoader,NodeLoader>
     (argv[1],argv[2],argv[3],argv[4],argv[5],&required_sequence, edgeloader, nodeloader);
   if (atleast4nodes){
-    int nb_nodes = (int)label.size();
+    nb_nodes = (int)label.size();
     map<int, int> label_to_node;
-    
+
     // Set the label mapping, necessary only to read the edges
     for (int i = 0; i < nb_nodes; i++)
       label_to_node[label[i]] = i;
@@ -554,23 +646,24 @@ int main( int argc, char** argv )
     WeightedDigraph G(2 * nb_nodes);
     
     for (int i = 0; i < (int)allEdges.size(); i++)
-      {
-	int u = label_to_node[allEdges[i].getFirst()]  +  (allEdges[i].label[0] == 'F' ? 0 : nb_nodes);
-	int v = label_to_node[allEdges[i].getSecond()] +  (allEdges[i].label[1] == 'F' ? 0 : nb_nodes);
-	int cost = strlen(seqs[ label_to_node[allEdges[i].getSecond()] ]) - (k_value - 1);
-	
-	G.adj_list[u].push_back(WeightedEdge(v, cost));
-      }
-    
+    {
+      int u = label_to_node[allEdges[i].getFirst()]  +  (allEdges[i].label[0] == 'F' ? 0 : nb_nodes);
+      int v = label_to_node[allEdges[i].getSecond()] +  (allEdges[i].label[1] == 'F' ? 0 : nb_nodes);
+      int cost = strlen(seqs[ label_to_node[allEdges[i].getSecond()] ]) - (k_value - 1);
+
+      G.adj_list[u].push_back(WeightedEdge(v, cost));
+    }
+
     // We add weights for the nodes only for convenience. All algorithms
     // work under the assumption of a edge-weighted directed graph.
     for (int i = 0; i < (int)G.adj_list.size(); i++)
       G.node_cost[i] = strlen(seqs[i % nb_nodes]) - (k_value - 1);
-    
+
     listAllBubbles(G, k_value, UL_MAX, LL_MAX, LL_MIN);
   }
   printSummary(stdout, nbBubbles);
 
+  for (unsigned int i=0; i<seqs.size(); i++) delete[] seqs[i]; // cleaning
   if (OUTPUT_PATH)
     fclose(path_output_file);
   fclose(seq_output_file_type0);
