@@ -213,7 +213,7 @@ int main(int argc, char **argv) {
             case 'o':
                 coherent_out = fopen(optarg, "w");
                 if(coherent_out == NULL){
-                    fprintf(stderr,"cannot open %s for writing results, exit\n", optarg);
+                    fprintf(stderr,"\nERROR: cannot open %s for writing results, exit\n", optarg);
                     exit(1);
                 }
                 printf(" Will output read-coherent results in file: %s\n", optarg);
@@ -221,7 +221,7 @@ int main(int argc, char **argv) {
             case 'u':
 		        uncoherent_out = fopen(optarg, "w");
                 if(uncoherent_out == NULL){
-                    fprintf(stderr,"cannot open %s for writing results, exit\n", optarg);
+                    fprintf(stderr,"\nERROR: cannot open %s for writing results, exit\n", optarg);
                     exit(1);
                 }
                 printf(" Will output unread-coherent results in file: %s\n", optarg);
@@ -368,7 +368,7 @@ int main(int argc, char **argv) {
 	
 	file=gzopen(toCheck_file,"r");
 	if(file == NULL){
-        fprintf(stderr,"cannot open file %s, exit\n",toCheck_file);
+        fprintf(stderr,"\nERROR: cannot open file %s, exit\n",toCheck_file);
         exit(1);
 	}
     if(!silent)
@@ -419,7 +419,7 @@ int main(int argc, char **argv) {
 #endif
     for (i=0;i<number_of_read_sets;i++){
         reads_files[i]=gzopen(reads_file_names[i],"r");
-        if(reads_files[i] == NULL){		fprintf(stderr,"cannot open reads file %s, exit\n",reads_file_names[i]);		exit(1);	}
+        if(reads_files[i] == NULL){		fprintf(stderr,"\nERROR: cannot open reads file %s, exit\n",reads_file_names[i]);		exit(1);	}
         
         if(!silent) printf("\nCheck read coherence... vs reads from %s (set %d)\n", reads_file_names[i], i);
         
